 //
//      Lab13_4: TMS320F28335
//      (c) Frank Bormann
//
//###########################################################################
//
// FILE:	Lab13_4.c
// 
// TITLE:	DSP28335 McBSP/SPI Communication  via F28335controlCARD 
//			and AT25256K at Peripheral Explorer Board
//
//		    EEPROM-Type: AT25C256K
//		    GPIO11:	CS - chip select for EEPROM
//			GPIO24_MDXB - used for SPISIMO
//			GPIO25_MDRB - used for SPISOMI
//			GPIO26_MCLKB - used for SPICLK
//			McBSP-B data rate =  1 MBit/s
//
//			GPIO15...12:  		4 Bit Hex-Encoder (0...15)
//			GPIO9,34,49:		LEDs LD1,LD3 and LD4
//
// 			Time - Base generated by CPU Timer 0 (100ms ISR)
//			Watchdog active , serviced in ISR and main-loop 
//
//			Aim of the test :  
//			1. Store the value from Hex-Encoder(GPIO15...12) into  
//			   EEPROM - Address 0x0040 when button PB1(GPIO17) is pushed
//			   
//			2. read back EEPROM address 0x0040 and display bits 3...0 of data
//			   at LED LD1...LD4 when button PB2(GPIO48) is pushed		 
//
//###########################################################################
//  Ver | dd mmm yyyy | Who  | Description of changes
// =====|=============|======|===============================================
//  3.0 | 02 Aug 2009 | F.B. | Lab13_4 for F28335; 
//  3.1 | 22 Nov 2009 | F.B  | Lab13_4 for F28335 @30MHz and PE revision 5
//###########################################################################
#include "DSP2833x_Device.h"
#include "IQmathLib.h"

// Symbols for GPIO-Pins 
#define WRITE_BUTTON 	GpioDataRegs.GPADAT.bit.GPIO17		// PB1 
#define READ_BUTTON  	GpioDataRegs.GPBDAT.bit.GPIO48		// PB2
#define CS_EEPROM	 	GpioDataRegs.GPADAT.bit.GPIO11	 	// CS - for EEPROM
	
// Commands for the EEPROM
#define WREN 0x06	// Write Enable
#define WRDI 0x04	// Write Disable
#define RDSR 0x05	// Read Status Register
#define WRSR 0x01	// Write Status Register
#define READ 0x03	// Read Command
#define WRITE 0x02	// Write Command 


// external function prototypes
extern void InitSysCtrl(void);
extern void InitPieCtrl(void);
extern void InitPieVectTable(void);
extern void InitCpuTimers(void);
extern void ConfigCpuTimer(struct CPUTIMER_VARS *, float, float);


// Prototype statements for functions found within this file.
void Gpio_select(void);
void McBSPB_Init(void);
int McBSP_B_EEPROM_Read_Status(void);
void McBSP_B_EEPROM_Write_Enable(void);
void McBSP_B_EEPROM_Write(int address,int data);
int McBSP_B_EEPROM_Read(int address);
interrupt void cpu_timer0_isr(void);

// Global Variables

//###########################################################################
//						main code									
//###########################################################################
void main(void)
{
	Uint16 write_control = 0;		// used to suppress write repetitions
	Uint16 data_read;				// return value from EEPROM-read

	InitSysCtrl();	// SYSCLK = 150 MHz, HSPCLK = 75 MHz,
					// LSPCLK = 37.5 MHz
					// all peripherals enabled
					// watchdog disabled
	EALLOW;
   	SysCtrlRegs.WDCR= 0x00AF;	// Re-enable the watchdog, Prescaler = 64 
   	EDIS;			

	Gpio_select();		// GPIO9, GPIO34 and GPIO49 as output
						// to 3 LEDs at Peripheral Explorer	
						// GPIO11 as output EEPROM-CS
						// GPIO17 and GPIO48 as inputs (PB1, PB2)

	McBSPB_Init();		// init McBSP-A as data channel for AIC23

	InitPieCtrl();		// basic setup of PIE table; from DSP2833x_PieCtrl.c
	
	InitPieVectTable();	// default ISR's in PIE

	EALLOW;
	PieVectTable.TINT0 = &cpu_timer0_isr;
	EDIS;

	InitCpuTimers();	// basic setup CPU Timer0, 1 and 2

	ConfigCpuTimer(&CpuTimer0,150,100000);
	// Enable CPU - Timer0 - INT in PIE: Group 1 interrupt 7
	PieCtrlRegs.PIEIER1.bit.INTx7 = 1;

	IER |= 1;			// INT1 enabled

	EINT;
	ERTM;

	CpuTimer0Regs.TCR.bit.TSS = 0;	// start timer0

	while (McBSP_B_EEPROM_Read_Status()&0x0001);// test for /RDY
    											// EEPROM ready for access

	while(1)
	{    
	  		while(CpuTimer0.InterruptCount == 0);
			CpuTimer0.InterruptCount = 0;
			
			EALLOW;
			SysCtrlRegs.WDKEY = 0x55;	// service WD #1
			EDIS;

			if (!WRITE_BUTTON && !write_control)	//  do an EEPROM write if PB1 is pushed
	    	{											
	    		write_control = 1;					// disable 2nd write until PB1 is released
	    		McBSP_B_EEPROM_Write_Enable();		// enable EEPROM for write
	       		while( !(McBSP_B_EEPROM_Read_Status() & 0x0002));	
	       											// wait for WEN = 1
	       		McBSP_B_EEPROM_Write(0x40,(GpioDataRegs.GPADAT.all>>12)&0xF);		 
		   											// write GPIO15-12 into EEPROM
		   		while (McBSP_B_EEPROM_Read_Status()&0x0001);
		   	}										// test for write in progress
	    	
			if (WRITE_BUTTON)
			{
				write_control = 0;		// PB1 is released; allow for new write
			}
			if (!READ_BUTTON)	  			// do an EEPROM-read if PB2 was pushed
			{								
	   		  data_read = McBSP_B_EEPROM_Read(0x40); // read value from EEPROM address 0x40  
	   		  GpioDataRegs.GPBDAT.bit.GPIO49 = (data_read & 4)>>2;  // bit 2
			  GpioDataRegs.GPBDAT.bit.GPIO34 = (data_read & 2)>>1;	// bit 1
			  GpioDataRegs.GPADAT.bit.GPIO9  = (data_read & 1);   	// bit 0
	    	 
			}								 

	}
} 

void Gpio_select(void)
{
	EALLOW;
	GpioCtrlRegs.GPAMUX1.all = 0;		// GPIO15 ... GPIO0 = General Puropse I/O
	GpioCtrlRegs.GPAMUX2.all = 0;		// GPIO31 ... GPIO16 = General Purpose I/O
	GpioCtrlRegs.GPBMUX1.all = 0;		// GPIO47 ... GPIO32 = General Purpose I/O
	GpioCtrlRegs.GPBMUX2.all = 0;		// GPIO63 ... GPIO48 = General Purpose I/O
	GpioCtrlRegs.GPCMUX1.all = 0;		// GPIO79 ... GPIO64 = General Purpose I/O
	GpioCtrlRegs.GPCMUX2.all = 0;		// GPIO87 ... GPIO80 = General Purpose I/O

	GpioCtrlRegs.GPAMUX2.bit.GPIO24 = 3; // GPIO24 = MDXB (SPISIMO)
	GpioCtrlRegs.GPAMUX2.bit.GPIO25 = 3; // GPIO25 = MDRB (SPISOMI)
	GpioCtrlRegs.GPAMUX2.bit.GPIO26 = 3; // GPIO26 = MCLKB (SPICLK)


	GpioCtrlRegs.GPADIR.all = 0;
	GpioCtrlRegs.GPADIR.bit.GPIO9 = 1;	// peripheral explorer: LED LD1 at GPIO9
	GpioCtrlRegs.GPADIR.bit.GPIO11 = 1;	// peripheral explorer: /CS EEPROM
	
	GpioCtrlRegs.GPBDIR.all = 0;		// GPIO63-32 as inputs
	GpioCtrlRegs.GPBDIR.bit.GPIO34 = 1;	// peripheral explorer: LED LD3 at GPIO34
	GpioCtrlRegs.GPBDIR.bit.GPIO49 = 1; // peripheral explorer: LED LD4 at GPIO49

	GpioCtrlRegs.GPCDIR.all = 0;		// GPIO87-64 as inputs

	GpioDataRegs.GPADAT.bit.GPIO11 = 1;	// set /CS - EEPROM to passive level	

	EDIS;
}  

//==============================================
// 					McBSP-B Setup
//  			SPI mode for AT25256
//==============================================
void McBSPB_Init()
{   	
    EALLOW;
   	McbspbRegs.SPCR1.all=0x0000;		// Reset Receiver
	McbspbRegs.SPCR1.bit.CLKSTP = 3;	// 3: Clockstop mode set to inactive low.
										// McBSP transmits data 1/2 cycle ahead of CLKX
	McbspbRegs.SPCR1.bit.RRST = 1;		// 1: Enables receiver

	McbspbRegs.SPCR2.all=0x0000;		// Reset FS generator, sample rate generator & transmitter
 	McbspbRegs.SPCR2.bit.FREE  = 1;		// Free run in break event 
	McbspbRegs.SPCR2.bit.GRST = 1;		// 1: Enables sample rate generator
	McbspbRegs.SPCR2.bit.XRST = 1;		// 1: Enables transmitter
	McbspbRegs.SPCR2.bit.FRST=1;		// 1: release frame logic from reset
	
	McbspbRegs.PCR.bit.CLKXP = 0;			// 0: Transmit data sampled on rising edge of CLKX
	McbspbRegs.PCR.bit.CLKRP = 1;			// 1: Receive data sampled on rising edge of MCLKR
	McbspbRegs.PCR.bit.CLKXM = 1;			// 1: McBSP is master in SPI-mode; 
											//    CLKX drives pin MCLKX and MCLKR(internally)
	McbspbRegs.PCR.bit.SCLKME = 0;			// 0: McBSP clock generator derived from LSPCLK 
	McbspbRegs.PCR.bit.FSXM = 1;			// 1: Transmit frame sync generated internally by CLKG
	McbspbRegs.PCR.bit.FSXP = 1;			// 1: Transmit frame sync pulses are active low

	McbspbRegs.SRGR1.bit.CLKGDV = 37;		// 37: CLKG = LSPCLK/(37+1) = 1 MHz on McBSP 

	McbspbRegs.SRGR2.bit.CLKSM = 1;			// 1: McBSP clock generator derived from LSPCLK
	McbspbRegs.SRGR2.bit.FSGM = 0;			// 0: McBSP generates a frame sync when DXR is copied into XSR

	McbspbRegs.XCR1.bit.XFRLEN1 = 0;		// 0: One word per frame for transmit
	McbspbRegs.XCR1.bit.XWDLEN1 = 0;		// 0: 8 bit transmit word
	
	McbspbRegs.XCR2.bit.XPHASE = 0;			// 0: Single phase transmit
	McbspbRegs.XCR2.bit.XDATDLY = 0;		// 0: No data delay between sync and first data bit

 	McbspbRegs.RCR1.bit.RFRLEN1 = 0;		// 0: One word per frame for receive
	McbspbRegs.RCR1.bit.RWDLEN1 = 0;		// 0: 8 bit receive word

	McbspbRegs.RCR2.bit.RDATDLY = 0;		// 0: No data delay because FSX not needed for chip select
	McbspbRegs.RCR2.bit.RPHASE = 0;			// 0: Single phase receive

    EDIS;
}

int McBSP_B_EEPROM_Read_Status(void)
{
	int k;
	CS_EEPROM  = 0;							// activate /CS of EEPROM 
	McbspbRegs.DXR1.all = RDSR;				// read status register command
	while (McbspbRegs.SPCR1.bit.RRDY == 0); // wait for end of SPI - cycle
	k=McbspbRegs.DRR1.all;					// dummy read to release receiver
	
	McbspbRegs.DXR1.all = 0;				// dummy data to drive 8 more SPICLK
	while (McbspbRegs.SPCR1.bit.RRDY == 0); // wait for end of SPI - cycle
	k=McbspbRegs.DRR1.all;					// read status , LSB is WIP
	CS_EEPROM = 1;							// deactivate /CS of EEPROM 
	return (k); 
}

void McBSP_B_EEPROM_Write_Enable(void)
{
	volatile int dummy;
	CS_EEPROM = 0;							// activate /CS for the EEPROM
	McbspbRegs.DXR1.all = WREN;				// write enable command
	while (McbspbRegs.SPCR1.bit.RRDY == 0); // wait for end of transmission
											// int-flag is reset when RXBUF was red
	dummy=McbspbRegs.DRR1.all;				// dummy read 
	CS_EEPROM = 1;							// deactivate /CS for the EEPROM 
}	

void McBSP_B_EEPROM_Write(int address,int data)
{
	volatile int dummy;
	CS_EEPROM = 0;							// activate /CS for the EEPROM
	McbspbRegs.DXR1.all = WRITE;			// send write code
	while (McbspbRegs.SPCR2.bit.XRDY == 0); // wait for end of transmission
	McbspbRegs.DXR1.all = (address>>8);		// write upper addressbyte
	while (McbspbRegs.SPCR2.bit.XRDY == 0); // wait for end of transmission
	McbspbRegs.DXR1.all = address;			// write lower addressbyte
	while (McbspbRegs.SPCR2.bit.XRDY == 0); // wait for end of transmission
	McbspbRegs.DXR1.all = data;				// send data
	while (McbspbRegs.SPCR2.bit.XRDY == 0); // Delays so that GPIO-11 is set
	dummy = McbspbRegs.DRR1.all;				// after the transfer
	while (McbspbRegs.SPCR1.bit.RRDY == 0);
	dummy = McbspbRegs.DRR1.all;
	CS_EEPROM = 1;							// deactivate /CS for the EEPROM 
}

int McBSP_B_EEPROM_Read(int address)
{
	int data;
	CS_EEPROM = 0;							// activate /CS for the EEPROM
	McbspbRegs.DXR1.all = READ;				// Read op-code
	while (McbspbRegs.SPCR2.bit.XRDY == 0);
	McbspbRegs.DXR1.all = address>>8;		// Higher byte of starting address
	while (McbspbRegs.SPCR2.bit.XRDY == 0);
	McbspbRegs.DXR1.all = address;			// Lower byte of starting address
	while (McbspbRegs.SPCR2.bit.XRDY == 0);
	McbspbRegs.DXR1.all = 0x00;				// send dummy Data
	while (McbspbRegs.SPCR2.bit.XRDY == 0);
	data = McbspbRegs.DRR1.all;				// Clears receive flag
	while (McbspbRegs.SPCR1.bit.RRDY == 0);
	data = McbspbRegs.DRR1.all;				// Reads data from memory
	CS_EEPROM = 1;							// deactivate /CS for the EEPROM 
	return(data);
}

interrupt void cpu_timer0_isr(void)
{
	CpuTimer0.InterruptCount++;
	EALLOW;
	SysCtrlRegs.WDKEY = 0xAA;	// service WD #2
	EDIS;
	// Acknowledge PIE-Group 1
	PieCtrlRegs.PIEACK.bit.ACK1 = 1;
}

//===========================================================================
// End of SourceCode.
//===========================================================================
